#include "gbox_applet.h"
#include "gbox_mailbox.h"
#include "gbox_menu.h"
#include "gbox_gfx.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <utime.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <signal.h>

MailBoxList *mb_list;
GboxApplet  *applet;
MailBox     *exchange_mb;
MailBox     *default_box;
GboxMenu    *mailbox_menu;

GtkWidget   *tip_list, *scroll_win;
static gint tt_state;

void
show_tooltip(GtkWidget *widget, GdkEvent *evt, gpointer user_data)
{
  GtkRequisition req, size_tip;
  GtkWidget      *box;
  gint x, y;

  if (tt_state) return; /* tooltip was activated by signal */
  
/*    stop_timer(); */
/*    update_tooltip(); */
/*    set_mail_icon(); */
/*    start_timer(); */
  
  gtk_clist_set_shadow_type(GTK_CLIST(tip_list), GTK_SHADOW_ETCHED_OUT);
  gtk_clist_set_column_auto_resize (GTK_CLIST(tip_list), 0, TRUE);
  
  gtk_clist_set_selection_mode(GTK_CLIST(tip_list), GTK_SELECTION_SINGLE);
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 0, 
			     gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 0));
  gtk_clist_set_column_width(GTK_CLIST(tip_list), 1, 
			     gtk_clist_optimal_column_width(GTK_CLIST(tip_list), 1));
  
  if (!applet->show_tooltip) return;

  stop_timer();
  
  gtk_widget_get_child_requisition(GTK_WIDGET(applet->gbox_applet), &req);
  gdk_window_get_origin(applet->gbox_applet->window, &x, &y);
  
  applet->tip = gtk_window_new(GTK_WINDOW_POPUP);
  gtk_container_set_border_width(GTK_CONTAINER(applet->tip), 0);

  box = gtk_hbox_new(0, 0);
  
  gtk_widget_show(GTK_WIDGET(tip_list));
  
  gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(tip_list), TRUE, TRUE, 0);

  gtk_widget_show(GTK_WIDGET(box));
  
  gtk_container_add(GTK_CONTAINER(applet->tip), GTK_WIDGET(box));

  gtk_widget_size_request(GTK_WIDGET(applet->tip), &size_tip);
  
  switch (applet->pos_tooltip) {
  case 0: /* ABOVE */
    if ((x + size_tip.width) > gdk_screen_width())
      x = gdk_screen_width() - size_tip.width;

    y -= size_tip.height;
    break;
  case 1: /* LEFT */
    if ((y + size_tip.height) > gdk_screen_height())
      y = gdk_screen_height() - size_tip.height;
    
    x -= 2*14;
    x -= size_tip.width;
    break;
  case 2: /* BELOW */
    if ((x + size_tip.width) > gdk_screen_width())
      x = gdk_screen_width() - size_tip.width;

    y += req.height;
    break;
  case 3: /* RIGHT */
    if ((y + size_tip.height) > gdk_screen_height())
      y = gdk_screen_height() - size_tip.height;
    
    x += req.width;
  }
  
  gtk_widget_popup(GTK_WIDGET(applet->tip), x, y);
  /* the next signal have to hide the tooltip */
  tt_state = 1; 
}

void
hide_tooltip(GtkWidget *widget, GdkEvent *evt, gpointer user_data)
{
  if (!tt_state) return;
  
  if (!applet->show_tooltip) return;
  gtk_widget_destroy(GTK_WIDGET(applet->tip));
  tip_list = NULL;
  tt_state = 0; /* just to be sure */
  
  start_timer();
}

void
update_tooltip()
{
  gchar     *text[2];
  MailBox   *walk = mb_list->first_mb;
  GdkColor  hilight_color = { 0, 0xCCDD, 0x1111, 0x1111 };
  
  if (!tip_list) tip_list = gtk_clist_new(2);
  gtk_clist_clear(GTK_CLIST(tip_list));
  
  switch (applet->style_tooltip) {
  case 0:
    text[0] = _("watched mailboxes");
    text[1] = g_strdup_printf("%d", gbox_mailbox_list_count(mb_list));
    gtk_clist_append(GTK_CLIST(tip_list), text);
    g_free(text[1]);
    
    text[0] = _("contain new mail");
    text[1] = g_strdup_printf("%d", gbox_mailbox_list_count_with_new_mail(mb_list));
    gtk_clist_append(GTK_CLIST(tip_list), text);
    g_free(text[1]);
    break;
  case 1:
    text[0] = _("highest priority");
    text[1] = g_strdup_printf("%d", mb_list->new_count[0]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    g_free(text[1]);
    
    text[0] = _("middle priority");
    text[1] = g_strdup_printf("%d", mb_list->new_count[1]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    g_free(text[1]);

    text[0] = _("lowest priority");
    text[1] = g_strdup_printf("%d", mb_list->new_count[2]);
    gtk_clist_append(GTK_CLIST(tip_list), text);
    g_free(text[1]);
    break;
  case 2:
    {
      int row_count = 0;
      
      while (walk) {
	text[0] = walk->nickname;
	text[1] = g_strdup_printf("%d", walk->mail_count);
      
	gtk_clist_append(GTK_CLIST(tip_list), text);
	g_free(text[1]);

	if (walk->mail_count)
	  gtk_clist_set_foreground(GTK_CLIST(tip_list), row_count, &hilight_color);
      
	gtk_clist_set_background(GTK_CLIST(tip_list), row_count, NULL);
	walk = walk->next;row_count++;
      }
      break;
    }
  case 3:
    while (walk) {
      if (walk->mail_count) {
	text[0] = walk->nickname;
	text[1] = g_strdup_printf("%d", walk->mail_count);
	
	gtk_clist_append(GTK_CLIST(tip_list), text);
	g_free(text[1]);
      }
      walk = walk->next;
    }
  }

  walk = mb_list->first_mb;
  while (walk) {
    gbox_menu_remove_box(mailbox_menu, walk);
    
    if (walk->show_in_menu)
      gbox_menu_add_box(mailbox_menu, walk);
      
    walk = walk->next;
  }
}

/*
  -3 - error
  -2 - size of mailbox became zero
  -1 - size of mailbox is less than before
   0 - size of mailbox is zero
   1 - size of mailbox is more than before
   2 - size of mailbox does not change but is not zero
   3 - new mail arrived in an empty mailbox
 */
int
check_mailbox(MailBox *mb)
{
  MailBox *walk = mb;
  struct stat buff;
  struct utimbuf timebuff;
  int    mail = 0;
  
  if (!stat(walk->mailfile, &buff)) {
    if (buff.st_size == walk->size)
      mail = (!walk->size?0:2);
    else {
      if (walk->size < buff.st_size) {
	if (!walk->size) mail = 3;
	else mail = 1;
      } else {
	if (walk->size > buff.st_size) {
	  if (!buff.st_size) mail = -2;
	  else mail = -1;
	}
      }
    }
    
    if (buff.st_mtime > walk->modification_time
	|| buff.st_size != walk->size)
      walk->mail_count = gbox_mailbox_count_mails(walk->mailfile,
						  NULL, FALSE);
    
    walk->modification_time = buff.st_mtime;
    walk->size = buff.st_size;
  } else {
    mail = -3;
  }
  
  /* Make sure other programs checking for new mail (like mutt) are not
     affected by our check. This must come after
     gbox_mailbox_count_mails(). */
  timebuff.actime = buff.st_atime;
  timebuff.modtime = buff.st_mtime;
  utime(walk->mailfile, &timebuff);

  if (walk->changed_last_time)
    (mb_list->new_count[walk->priority])--;
  
  if (walk->mail_count > 0) {
    (mb_list->new_count[walk->priority])++;
    if (!walk->changed_last_time) {
      if (applet->style_beep)
	gdk_beep();
    }    
    walk->changed_last_time = TRUE;
  } else
    walk->changed_last_time = FALSE;

  applet->mail_count_num = gbox_mailbox_list_count_sum(mb_list, applet->mail_count_prio);
  
/*    stop_timer(); */
  update_tooltip();
  set_mail_icon();
/*    start_timer(); */
  return mail;
}

void
update_mailboxes_cb(AppletWidget *app, gpointer data)
{
  MailBox * walk = mb_list->first_mb;
  
  while (walk) {
    check_mailbox(walk);
    walk = walk->next;
  }

  set_mail_icon();
}

static void 
draw_image (GtkWidget *darea, GdkEventExpose *event, gpointer data)
{
  GdkPixbuf *pixbuf;
  GtkStyle *style;
  GdkFont  *old_font = NULL;
  gchar    *label;
  int      h;
  
  pixbuf = (GdkPixbuf *)gtk_object_get_data(GTK_OBJECT(darea), "pixbuf");

  if (!gdk_pixbuf_get_pixels(pixbuf)) {
    g_warning ("art_pixbuf is NULL in expose_func!!\n");
    return;
  }

  gdk_window_clear_area (darea->window,
			 event->area.x,
			 event->area.y,
			 event->area.width,
			 event->area.height);
  
  if (gdk_pixbuf_get_has_alpha(pixbuf)) {
    gdk_pixbuf_render_to_drawable_alpha (pixbuf,
					 darea->window,
					 0, 0, 0, 0,
					 event->area.width, 
					 event->area.height,
					 GDK_PIXBUF_ALPHA_FULL,
					 255,
					 GDK_RGB_DITHER_MAX,
					 0, 0);    
  } else {
    gdk_pixbuf_render_to_drawable (pixbuf,
				   darea->window,
				   darea->style->black_gc,
				   0, 0, 0, 0,
				   event->area.width, 
				   event->area.height,
				   GDK_RGB_DITHER_NORMAL,
				   0, 0);    
  }

  style = gtk_widget_get_style(GTK_WIDGET(applet->draw_area));

  /* show sum of mails??? */
  if (applet->show_mail_count) {
    if (applet->font_name) {
      old_font = style->font;
      style->font = gdk_font_load(applet->font_name);
    }
    label = g_strdup_printf("%d", applet->mail_count_num);
  } else
    label = g_strdup("");
  
  /* draw sum of mails */
  h = gdk_string_height(style->font, label);
  
  gtk_paint_string (style,
		    applet->draw_area->window,
		    GTK_STATE_NORMAL,
		    NULL,
		    applet->draw_area,
		    "label",
		    applet->mail_count_x_pos,
		    applet->mail_count_y_pos + h,
		    label);

  if (applet->show_mail_count && applet->font_name) {
    gdk_font_unref(style->font);
    style->font = old_font;
  }
  
  g_free(label);
}

void
set_mail_icon()
{
  int      i;
  
  for (i = 0; i < COUNT_PRIORITY; i++)
    if (mb_list->new_count[i] > 0) {
      if (applet->current_prio != i) {
	gbox_gfx_draw(applet->new_email_image[i]);
	gbox_gfx_expose(applet->new_email_image[i], draw_image);
	applet->current_prio = i;
      }
      return;
    }

  if (applet->current_prio != -1) { /* no email icon */
    gbox_gfx_draw(applet->no_email_image);
    gbox_gfx_expose(applet->no_email_image, draw_image);
    applet->current_prio = -1;
  }
}

gint
timeout_callback( gpointer data )
{
  check_mailbox((MailBox *) data);
  
  set_mail_icon();
  
  return 1; /* 0 will stop the timer forever */
}

static gint
exec_mail_prog(GtkWidget *widget, GdkEventButton *evt)
{
  /* if it is not the left button -> exit */
  if (!(evt->button == 1)) return 0;
  
  if (applet->mail_prog && (strlen(applet->mail_prog)) > 0)
    gnome_execute_shell(NULL, applet->mail_prog);
  
  return 1;
}

void
start_timer()
{
  MailBox *walk = mb_list->first_mb;
    
  while (walk) {
    check_mailbox(walk);
    walk->timer = gtk_timeout_add( walk->interval,
				   timeout_callback,
				   walk);
    walk = walk->next;
  }
}

void
stop_timer()
{
  MailBox *walk = mb_list->first_mb;
    
  while (walk) {
    gtk_timeout_remove(walk->timer);
    walk = walk->next;
  }
}

static void
mailcheck_destroy (GtkWidget *widget, gpointer data)
{
  gtk_main_quit();
}

static GtkWidget *
create_mail_widgets (GtkWidget *parent, GtkWidget **event_box)
{
  GtkWidget *box;
  /*  GtkStyle       *style;*/
  int       i;
  
  applet->gbox_frame = gtk_frame_new(NULL);

  gtk_frame_set_shadow_type(GTK_FRAME(applet->gbox_frame), 
			    (applet->show_frame?GTK_SHADOW_ETCHED_IN:GTK_SHADOW_NONE));
  
  box = gtk_event_box_new ();
  gtk_signal_connect (GTK_OBJECT (box), "destroy",
		      GTK_SIGNAL_FUNC (mailcheck_destroy), NULL);
  
  //  gtk_widget_show (box);
  
  if (!g_file_test(gbox_gfx_file(applet->no_email_image), G_FILE_TEST_ISFILE)) {
    printf(_("Could not find the specified image-file (%s)\n"), 
	   gbox_gfx_file(applet->no_email_image));
    exit(1);
  }
    
  /* create drawing area */
  applet->draw_area = gtk_drawing_area_new();
  applet->fixed_box = gtk_fixed_new();

  /* set drawing widget */
  gbox_gfx_set_widget(applet->no_email_image, applet->draw_area);
  
  for (i = 0; i < COUNT_PRIORITY; i++) {
    gbox_gfx_set_widget(applet->new_email_image[i], 
			applet->draw_area);
  }
  
  /* resize fixed widget to first image -- Oops?!?!?! */
  gtk_widget_set_usize(GTK_WIDGET(applet->fixed_box), 
		       gbox_gfx_width(applet->new_email_image[0]), 
		       gbox_gfx_height(applet->new_email_image[0]));
  
  gtk_container_add(GTK_CONTAINER(applet->gbox_frame), box);
  gtk_container_add(GTK_CONTAINER(box), applet->fixed_box);

  gtk_fixed_put(GTK_FIXED(applet->fixed_box), applet->draw_area, 0, 0); 

  gtk_signal_connect( GTK_OBJECT(box), "button_release_event",
		      GTK_SIGNAL_FUNC(exec_mail_prog), 
		      NULL);
  gtk_signal_connect (GTK_OBJECT(applet->draw_area), "expose_event",
					   GTK_SIGNAL_FUNC (draw_image), NULL);
  
  gtk_widget_show(applet->draw_area);
  gtk_widget_show(box);
  gtk_widget_show(applet->fixed_box);
  gtk_widget_show(applet->gbox_frame);

  *event_box = box;
  
  return applet->gbox_frame;
}

static void
about_cb(AppletWidget* w, gpointer cbdata)
{
  GtkWidget* about_box, *l, *hbox;
  
  static const gchar* author[] = { N_("Andreas Bsching (crunchy@tzi.de) - programming"), 
				   N_("Thomas Buntrock (bunti@tzi.de) - graphics & programming"), 
				   NULL };
  

  about_box = gnome_about_new(_(PACKAGE), VERSION,
			      "(C) 1999 Andreas Bsching",
			      author,
			      _("Watching multiply mbox files ..."),
			      NULL);

  hbox = gtk_hbox_new (TRUE, 0);
  l = gnome_href_new ("http://gbox-applet.sourceforge.net/",
		      _("Gbox Applet Home Page"));
  gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about_box)->vbox),
		      hbox, TRUE, FALSE, 0);
  gtk_widget_show_all (hbox);
  
  gtk_widget_show(about_box);
}

static void
save_session_info(void)
{
  MailBox *walk = mb_list->first_mb;
  gchar box[PATH_MAX];
  int i;
  
  gnome_config_push_prefix(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  gnome_config_set_string("/gbox_applet/global/no_mail_pixmap", 
			  gbox_gfx_file(applet->no_email_image));
  gnome_config_set_string("/gbox_applet/global/mail_prog", applet->mail_prog);
  gnome_config_set_bool("/gbox_applet/global/beep", applet->style_beep);
  gnome_config_set_bool("/gbox_applet/global/show_frame", applet->show_frame);
  gnome_config_set_bool("/gbox_applet/global/show_mail_count", applet->show_mail_count);
  gnome_config_set_int("/gbox_applet/global/mail_count_prio", applet->mail_count_prio);
  gnome_config_set_int("/gbox_applet/global/mail_count_x_pos", applet->mail_count_x_pos);
  gnome_config_set_int("/gbox_applet/global/mail_count_y_pos", applet->mail_count_y_pos);
  if (applet->font_name)
    gnome_config_set_string("/gbox_applet/global/fontname", applet->font_name);
  gnome_config_set_bool("/gbox_applet/global/tooltip", applet->show_tooltip);
  gnome_config_set_int("/gbox_applet/global/tooltip_style", applet->style_tooltip);
  gnome_config_set_int("/gbox_applet/global/tooltip_pos", applet->pos_tooltip);
  gnome_config_set_int("/gbox_applet/global/count_mailbox", gbox_mailbox_list_count(mb_list));

  gnome_config_set_string("/gbox_applet/default_mailbox/filename", default_box->mailfile);
  gnome_config_set_string("/gbox_applet/default_mailbox/nickname", default_box->nickname);
  gnome_config_set_int("/gbox_applet/default_mailbox/interval", default_box->interval);
  gnome_config_set_int("/gbox_applet/default_mailbox/priority", default_box->priority);
  gnome_config_set_int("/gbox_applet/default_mailbox/menu_type", default_box->menu_type);
  gnome_config_set_bool("/gbox_applet/default_mailbox/show_in_menu", default_box->show_in_menu);

  for (i = 0;i < gbox_mailbox_list_count(mb_list); i++) {
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/filename", i);
    gnome_config_set_string(box, walk->mailfile);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
    gnome_config_set_string(box, walk->nickname);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/interval", i);
    gnome_config_set_int(box, walk->interval);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/priority", i);
    gnome_config_set_int(box, walk->priority);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/show_in_menu", i);
    gnome_config_set_bool(box, walk->show_in_menu);
    snprintf(box, sizeof(box), "/gbox_applet/mailbox%d/menu_type", i);
    gnome_config_set_int(box, walk->menu_type);
    walk = walk->next;
  }
  
  gnome_config_set_int("/gbox_applet/priority/count", COUNT_PRIORITY);
  for (i = 0;i < COUNT_PRIORITY; i++) {
    snprintf(box, sizeof(box), "/gbox_applet/priority/prio-%d", i);
    gnome_config_set_string(box, 
			    gbox_gfx_file(applet->new_email_image[i]));
  }
  
  gnome_config_pop_prefix();
  gnome_config_sync();
  gnome_config_drop_all();
}

static void
quit_applet(GtkWidget* w, gpointer cbdata)
{
  save_session_info();
  applet_widget_gtk_main_quit();
}

static gint
applet_save_session(GtkWidget* w, const gchar* priv, const gchar* global)
{
  save_session_info();
  return FALSE;
}

static void
read_config_data(gchar* cfgpath)
{
  gchar   *tmp, *nick, *help;
  int     count_mb = 0, i, num, prio, count_prio = 0, show, menu_type;
  char    box[128];
  char    pixmaps[1024];
  MailBox *mailbox;
   
  if(cfgpath == NULL)
    return;

  gnome_config_push_prefix(cfgpath);
   
  snprintf(pixmaps, sizeof(pixmaps), 
	   "/gbox_applet/global/no_mail_pixmap=%s", 
	   GBOX_PIXMAP_DIR "/default-no-mail.png");


  tmp = gnome_config_get_string(pixmaps);
  printf("PIXMAP: %s\n", tmp);
   
  applet->no_email_image = gbox_gfx_new(tmp, NULL);
  g_free(tmp);

  tmp = gnome_config_get_string("/gbox_applet/global/mail_prog=emacs -f gnus");
  applet->mail_prog = g_strdup(tmp);
  g_free(tmp);

  applet->style_beep = gnome_config_get_bool("/gbox_applet/global/beep=false");
  applet->show_frame = gnome_config_get_bool("/gbox_applet/global/show_frame=true");   
  applet->show_mail_count = gnome_config_get_bool("/gbox_applet/global/show_mail_count=false");   
  applet->mail_count_prio = gnome_config_get_int("/gbox_applet/global/mail_count_prio=2");   
  applet->mail_count_x_pos = gnome_config_get_int("/gbox_applet/global/mail_count_x_pos=0");   
  applet->mail_count_y_pos = gnome_config_get_int("/gbox_applet/global/mail_count_y_pos=0");   
  tmp = gnome_config_get_string("/gbox_applet/global/fontname");
  if (tmp) {
    applet->font_name = g_strdup(tmp);
    g_free(tmp);
  } else
    applet->font_name = NULL;

  applet->show_tooltip = gnome_config_get_bool("/gbox_applet/global/tooltip=false");   
  applet->style_tooltip = gnome_config_get_int("/gbox_applet/global/tooltip_style=0");
  applet->pos_tooltip = gnome_config_get_int("/gbox_applet/global/tooltip_pos=0");
  count_mb = gnome_config_get_int("/gbox_applet/global/count_mailbox=0");

  tmp = gnome_config_get_string("/gbox_applet/default_mailbox/filename");
  default_box->mailfile = g_strdup(tmp);
  g_free(tmp);
  tmp = gnome_config_get_string("/gbox_applet/default_mailbox/nickname");
  default_box->nickname = g_strdup(tmp);
  g_free(tmp);
  default_box->interval = gnome_config_get_int("/gbox_applet/default_mailbox/interval=10");
  default_box->show_in_menu = gnome_config_get_bool("/gbox_applet/default_mailbox/show_in_menu=false");
  default_box->menu_type = gnome_config_get_int("/gbox_applet/default_mailbox/menu_type=0");
  default_box->priority = gnome_config_get_int("/gbox_applet/default_mailbox/priority=2");

  for (i = 0; i < count_mb; i++) {
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/filename", i);
    tmp = gnome_config_get_string(box);
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/nickname", i);
    nick = gnome_config_get_string(box);
    if (!nick) {
      help = strrchr(tmp, '/');
      if (help) nick = help + 1;
    }
     
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/interval", i);
    num = gnome_config_get_int(box);
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/show_in_menu=false", i);
    show = gnome_config_get_bool(box);
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/menu_type=0", i);
    menu_type = gnome_config_get_int(box);
    snprintf(box, (size_t) sizeof(box), "/gbox_applet/mailbox%d/priority=%d", i, COUNT_PRIORITY - 1);
    prio = gnome_config_get_int(box);
     
    mailbox = gbox_mailbox_new(tmp, nick, num, prio, show, menu_type);
     
    gbox_mailbox_add(mb_list, mailbox);
  }

  snprintf(box, (size_t) sizeof(box), "/gbox_applet/priority/count=%d", COUNT_PRIORITY);
  count_prio = gnome_config_get_int(box);
  for (i = 0; i < count_prio; i++) {
    snprintf(box, (size_t) sizeof(box), "/default-prio-%d.png", i);
    snprintf(pixmaps, sizeof(pixmaps), 
	     "/gbox_applet/priority/prio-%d=%s%s", i, 
	     GBOX_PIXMAP_DIR, box);
    tmp = gnome_config_get_string(pixmaps);
    applet->new_email_image[i] = gbox_gfx_new(tmp, NULL);
    g_free(tmp);
  }
   
  gnome_config_pop_prefix();
}

void
sigusr1(int no)
{
  (tt_state ? hide_tooltip(NULL, NULL, NULL) : show_tooltip(NULL, NULL, NULL));
  
  signal(SIGUSR1, sigusr1);
}

int
main(int argc, char *argv[])
{
  int i;
  GtkWidget *frame;
  
  applet = (GboxApplet *)malloc(sizeof(GboxApplet));
  applet->prop_box_active = FALSE;
  applet->font_name = NULL;
  applet->current_prio = -2;
  tt_state = 0;
  
  exchange_mb = (MailBox *)malloc(sizeof(MailBox));
  exchange_mb->mailfile = NULL;
  exchange_mb->nickname = NULL;

  default_box = (MailBox *)malloc(sizeof(MailBox));
  default_box->mailfile = NULL;
  default_box->nickname = NULL;

  mb_list = gbox_mailbox_list_new();
  
  mb_list->first_mb = NULL;
  mb_list->last_mb = NULL;
  tip_list = NULL;

  for (i = 0; i < COUNT_PRIORITY; i++)
    mb_list->new_count[i] = 0;
  
  bindtextdomain(PACKAGE, GNOMELOCALEDIR);
  textdomain(PACKAGE);
  
  applet_widget_init(PACKAGE, VERSION, argc, argv,
		     NULL, 0, NULL); 

  /* init drawing things */
  gdk_rgb_set_verbose (TRUE);
  gdk_rgb_init ();
  gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
  gtk_widget_set_default_visual (gdk_rgb_get_visual ());

  /* create applet */
  applet->gbox_applet = applet_widget_new(PACKAGE);
  if (!applet->gbox_applet) {
    printf("failed to create applet!\n");
    exit(0);
  }
  gtk_widget_realize(applet->gbox_applet);
  
  /* reading configuration */
  read_config_data(APPLET_WIDGET(applet->gbox_applet)->privcfgpath);
  
  frame = create_mail_widgets(applet->gbox_applet, &applet->mailcheck);
  //  gtk_widget_show(applet->mailcheck);
  applet_widget_add (APPLET_WIDGET (applet->gbox_applet), frame);
  
  gtk_widget_show_all(applet->gbox_applet);
  
  mailbox_menu = gbox_menu_new(APPLET_WIDGET(applet->gbox_applet));

  /* tooltip */
  start_timer();
  update_mailboxes_cb(NULL, NULL);
  update_tooltip();  
  set_mail_icon();
  
  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "delete_event",
		     GTK_SIGNAL_FUNC(quit_applet), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->gbox_applet), "save_session",
		     GTK_SIGNAL_FUNC(applet_save_session), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->mailcheck), "enter_notify_event",
		     GTK_SIGNAL_FUNC(show_tooltip), NULL);
  gtk_signal_connect(GTK_OBJECT(applet->mailcheck), "leave_notify_event",
		     GTK_SIGNAL_FUNC(hide_tooltip), NULL);

  applet_widget_register_stock_callback (APPLET_WIDGET(applet->gbox_applet),
					 "properties",
					 GNOME_STOCK_MENU_PROP,
					 _("properties"),
					 applet_properties,
					 NULL);
  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"update",
					GNOME_STOCK_MENU_REFRESH,
					_("update"),
					update_mailboxes_cb, NULL);
  applet_widget_register_stock_callback(APPLET_WIDGET(applet->gbox_applet),
					"about",
					GNOME_STOCK_MENU_ABOUT,
					_("About..."),
					about_cb, NULL);
  
  signal(SIGUSR1, sigusr1);
  applet_widget_gtk_main(); 
  
  /* free some memory */
  gbox_menu_free(mailbox_menu);
  gbox_mailbox_list_free(mb_list);

  /* free images */
  gbox_gfx_free(applet->no_email_image);
  for (i = 0; i < COUNT_PRIORITY; i ++)
    gbox_gfx_free(applet->new_email_image[i]);
  
  free(mb_list);
  free(exchange_mb);
  free(default_box);
  free(applet);
  
  return 0;
}

