#include "gbox_applet.h"
#include "gbox_mailbox.h"

#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/file.h>
#include <fcntl.h>
#include <sys/types.h>

MailBoxList *
gbox_mailbox_list_new()
{
  MailBoxList *list;
  int         i;
  
  list = (MailBoxList *) g_malloc(sizeof(MailBoxList));
  list->first_mb = NULL;
  list->last_mb = NULL;
  list->box_count = 0;
  for (i = 0; i < COUNT_PRIORITY; i++) list->new_count[i] = 0;
  return list;
}

MailBoxList *
gbox_mailbox_list_copy(MailBoxList *list)
{
  MailBoxList *new_list = gbox_mailbox_list_new();
  MailBox     *walk = list->first_mb;
  int         i;
  
  if (!list) {
    gbox_mailbox_list_free(new_list);
    printf("Ooops!\n");
    
    return NULL;
  }
    
  while (walk) {
    gbox_mailbox_add(new_list, gbox_mailbox_copy(walk));
    walk = walk->next;
  }

  for (i = 0; i < COUNT_PRIORITY; i++) new_list->new_count[i] = list->new_count[i];

  return new_list;
}


void
gbox_mailbox_list_free(MailBoxList *list)
{
  while (list->first_mb)
    gbox_mailbox_remove(list, list->first_mb->mailfile);
  
  g_free(list);
}

MailBox *
gbox_mailbox_new(char *file, char *nick, int interval, int prio, int show, int menu_type)
{
  MailBox *new_mb = g_malloc(sizeof(MailBox));
  
  new_mb->next = NULL;
  new_mb->mailfile = g_strdup(file);
  new_mb->nickname = g_strdup(nick);
  new_mb->size = 0;
  new_mb->interval = interval;
  new_mb->priority = prio;
  new_mb->show_in_menu = show;
  new_mb->menu_type = menu_type;
  new_mb->mail_count = 0;
  new_mb->modification_time = 0;
  new_mb->changed_last_time = FALSE;
  
  return new_mb;
}

MailBox *
gbox_mailbox_copy(MailBox *box)
{
  MailBox *new_box = g_malloc(sizeof(MailBox));
  
  new_box->next = NULL;
  new_box->mailfile = g_strdup(box->mailfile);
  new_box->nickname = g_strdup(box->nickname);
  new_box->size = box->size;
  new_box->interval = box->interval;
  new_box->priority = box->priority;
  new_box->show_in_menu = box->show_in_menu;
  new_box->menu_type = box->menu_type;
  new_box->mail_count = box->mail_count;
  new_box->changed_last_time = box->changed_last_time;
  
  return new_box;
}

void
gbox_mailbox_add(MailBoxList *list, MailBox *new_box)
{
  if (list->last_mb) list->last_mb->next = new_box;
  list->last_mb = new_box;
  if (!list->first_mb) list->first_mb = new_box;
  
  list->box_count++;
}

void
gbox_mailentries_free(MailEntry **mail)
{
  MailEntry *current = NULL;
  
  if ((*mail) == NULL) return;

  while ((*mail) != NULL) {
    current = (*mail);
    g_free ((*mail)->from);
    g_free ((*mail)->subject);
    (*mail) = (*mail)->next;
    g_free (current);    
  }
}

int
gbox_mailbox_count_mails(char *file, MailEntry **mail, int with_data)
{
/*    struct flock ld; */
/*    int fd; */
/*    FILE *f = 0; */
/*    char buf[1024]; */
/*    char *boundary = NULL; */
/*    char *boundary_end = NULL; */
/*    char *pos1 = NULL, *pos2 = NULL; */
/*    int count = 0; */
/*    int ignore = 0; */
/*    int in_header = 1; */
/*    char *lf; */
/*    MailEntry *newMail = NULL; */
/*    MailEntry *lastMail = NULL; */
/*    MailEntry *prevMail = NULL; */
 
/*    ld.l_type = F_RDLCK; */
/*    ld.l_whence = SEEK_SET; */
/*    ld.l_start = 0; */
/*    ld.l_len = 0; */
  
/*    if (with_data)  */
/*      (*mail) = NULL; */
  
/*    if ((fd = open(file,O_RDONLY)) != -1) { */
/*      while (fcntl (fd, F_SETLKW, &ld) == -1) ; */
    
/*      if ((f = fdopen(fd,"r"))) { */
/*        while (fgets(buf,sizeof(buf),f) != 0) { */
/*  	if (!ignore && strncmp(buf,"From ",5) == 0){ */
/*  	  count++; */
/*  	  in_header = 1; */
	  
/*  	  if (with_data){ */
/*  	    newMail = (MailEntry*)malloc(sizeof(MailEntry)); */
/*  	    newMail->from = newMail->subject = NULL; */
/*  	    newMail->next = NULL; */
	    			
/*  	    if (lastMail!=NULL){ */
/*  	      lastMail->next=newMail; */
/*  	      prevMail=lastMail; */
/*  	    } */
	    
/*  	    lastMail=newMail; */
/*  	    if ((*mail)==NULL) */
  //  	      (*mail) = newMail; /*set head of list for passing back*/
/*  	  } */
/*  	} */
	
/*  	if (in_header && with_data && strncmp(buf,"From: ",6) == 0) { */
  	  /* extract From */
/*  	  lf = strchr(buf,'\n'); */
/*  	  if (lf != NULL) *lf = '\0'; */
/*  	  lastMail->from=(char*)malloc((size_t)(strlen(buf)-5)); */
/*  	  strcpy(lastMail->from,buf+6); */
/*  	  printf("found From: %s\n", lastMail->from); */
/*  	} */
	  
/*  	if (in_header && with_data && strncmp(buf,"Subject: ",9) == 0) { */
	  /* extract Subject */
/*  	  lf = strchr(buf,'\n'); */
/*  	  if (lf != NULL) *lf = '\0'; */
/*  	  lastMail->subject=(char*)malloc((size_t)(strlen(buf)-8)); */
/*  	  strcpy(lastMail->subject,buf+9); */
/*  	  printf("found Subject: %s\n", lastMail->subject); */
/*  	} */
		
/*  	if (!ignore && strncmp(buf,"Content-Type: ",14) == 0){ */
/*  	  if ((pos1 = strstr(buf,"boundary=")) != 0) { */
  	    /* boundary in the same line */ 
/*  	    pos1 += 9; */
/*  	    ignore = 1; */
	    
/*  	    pos2 = pos1 + 1; */
/*  	    if (pos1[0] == '"') */
/*  	      while (*pos2) if (*pos2 != '"') pos2++; else break; */
/*  	    else */
/*  	      while (*pos2) if (*pos2 != '\n' && *pos2 != ' ') pos2++; else break; */

/*  	    if (pos1 != 0 && pos2 > pos1+1) { */
/*  	      boundary = (char*)malloc((size_t)(pos2-pos1 + 1)); */
/*  	      strncpy(boundary,pos1+1,pos2-pos1-1); */
/*  	      boundary[pos2-pos1-1] = '\0'; */
/*  	      boundary_end = (char*)malloc((size_t)(pos2-pos1+4)); */
/*  	      sprintf(boundary_end, "--%s--", boundary); */
/*  	    } */
	    
/*  	  } else { */
  	    /* boundary in the next line */  
/*  	    if (fgets(buf,sizeof(buf),f) != 0) { */
/*  	      if ((pos1 = strstr(buf,"boundary=")) != 0) {   */
/*  		pos1 += 9; */
/*  		ignore = 1; */
/*  		pos2 = pos1 + 1; */
/*  		if (pos1[0] == '"') */
/*  		  while (*pos2) if (*pos2 != '"') pos2++; else break; */
/*  		else */
/*  		  while (*pos2) if (*pos2 != '\n' && *pos2 != ' ') pos2++; else break; */
		  
/*  		if (pos1 != 0 && pos2 > pos1+1) { */
/*  		  boundary = (char*)malloc((size_t)(pos2-pos1 + 1)); */
/*  		  strncpy(boundary,pos1+1,pos2-pos1-1); */
/*  		  boundary[pos2-pos1-1] = '\0'; */
/*  		  boundary_end = (char*)malloc((size_t)(pos2-pos1+5)); */
/*  		  sprintf(boundary_end, "--%s--", boundary); */
/*  		} */
/*  	      } */
/*  	    } */
/*  	  } */
/*  	} */
	
/*  	if (in_header && strncmp(buf, "Status: ", 8) == 0) { */
/*  	  if (strchr(buf, 'R')) { */
/*  	    if (count) */
/*  	      count--; */
/*  	    if (with_data){ */
/*  	      if (*mail == lastMail) */
/*  		*mail = NULL; */
/*  	      if (lastMail->from!=NULL) */
/*  	      	free(lastMail->from); */
/*  	      if (lastMail->subject!=NULL) */
/*  	        free(lastMail->subject); */
/*  	      free(lastMail); */
/*  	      if (prevMail!=NULL){ */
/*  	        lastMail=prevMail; */
/*  	      	lastMail->next=NULL; */
/*  	      } else { */
/*  	        lastMail=NULL; */
/*  	      } */
/*  	    } */
/*  	  } */
/*  	} */
	
/*  	if ((ignore == 1) && (strncmp(buf,boundary_end,strlen(boundary_end)) == 0)) { */
/*  	  ignore = 0; */
/*  	  free(boundary); boundary = NULL; */
/*  	  free(boundary_end); boundary_end = NULL; */
/*  	} */

/*  	if (in_header && (strlen(buf) == 0 || buf[0] == '\n'))  */
/*  	  in_header = 0; */

/*  	memset(buf, 0, sizeof(buf)); */
/*        } */
/*        fclose (f); */
/*      } else count = -1; */
/*      ld.l_type = F_UNLCK; */
/*      while (fcntl (fd, F_SETLKW, &ld) != -1) ;     */
/*      close(fd); */
/*    }else count = -1; */
/*    if (boundary) { */
/*  	  free(boundary); boundary = NULL; */
/*  	  free(boundary_end); boundary_end = NULL; */
/*    } */

/*    return count; */
  struct flock ld;
  int fd;
  FILE *f = 0;
  char buf[1024];
  int count = 0;
  int in_header = 1;
  char *lf;
  MailEntry *newMail = NULL;
  MailEntry *lastMail = NULL;
  MailEntry *prevMail = NULL;
 
  ld.l_type = F_RDLCK;
  ld.l_whence = SEEK_SET;
  ld.l_start = 0;
  ld.l_len = 0;
  
  if (with_data) 
    (*mail) = NULL;
  
  if ((fd = open(file,O_RDONLY)) != -1) {
    while (fcntl (fd, F_SETLKW, &ld) == -1) ;
    
    if ((f = fdopen(fd,"r"))) {
      while (fgets(buf,sizeof(buf),f) != 0) {
	if (strncmp(buf,"From ",5) == 0){
	  count++;
	  in_header = 1;
	  
	  if (with_data){
	    newMail = (MailEntry*)malloc(sizeof(MailEntry));
	    newMail->from = newMail->subject = NULL;
	    newMail->next = NULL;
	    			
	    if (lastMail!=NULL){
	      lastMail->next=newMail;
	      prevMail=lastMail;
	    }
	    
	    lastMail=newMail;
	    if ((*mail)==NULL)
	      (*mail) = newMail; /*set head of list for passing back*/
	  }
	}
	
	if (in_header && with_data && strncmp(buf,"From: ",6) == 0) {
	  /* extract From */
	  lf = strchr(buf,'\n');
	  if (lf != NULL) *lf = '\0';
	  lastMail->from=(char*)malloc((size_t)(strlen(buf)-5));
	  strcpy(lastMail->from,buf+6);
	}
	  
	if (in_header && with_data && strncmp(buf,"Subject: ",9) == 0) {
	  /* extract Subject */
	  lf = strchr(buf,'\n');
	  if (lf != NULL) *lf = '\0';
	  lastMail->subject=(char*)malloc((size_t)(strlen(buf)-8));
	  strcpy(lastMail->subject,buf+9);
	}

	if (in_header && strncmp(buf, "Status: ", 8) == 0) {
	  if (strchr(buf, 'R')) {
	    if (count)
	      count--;
	    if (with_data){
	      if (*mail == lastMail)
		*mail = NULL;
	      if (lastMail->from!=NULL)
	      	free(lastMail->from);
	      if (lastMail->subject!=NULL)
	        free(lastMail->subject);
	      free(lastMail);
	      if (prevMail!=NULL){
	        lastMail=prevMail;
	      	lastMail->next=NULL;
	      } else {
	        lastMail=NULL;
	      }
	    }
	  }
	}

	if (in_header && (strlen(buf) == 0 || buf[0] == '\n')) 
	  in_header = 0;

	memset(buf, 0, sizeof(buf));
      }
      fclose (f);
    } else count = -1;
    ld.l_type = F_UNLCK;
    while (fcntl (fd, F_SETLKW, &ld) != -1) ;    
    close(fd);
  } else
    count = -1;

  return count;
}

int
gbox_mailbox_list_count_with_new_mail(MailBoxList *list)
{
  MailBox *walk = list->first_mb;
  int count = 0;
  
  while (walk) {
    if (walk->mail_count) count++;
    walk = walk->next;
  }
  
  return count;
}
 
int
gbox_mailbox_list_count_sum(MailBoxList *list, int prio) {
  MailBox *walk = list->first_mb;
  int count = 0;
  
  while (walk) {
    if (walk->mail_count && walk->priority <= prio) count+=walk->mail_count;
    walk = walk->next;
  }
  
  return count;  
}

int
gbox_mailbox_list_count(MailBoxList *list)
{
  MailBox *walk = list->first_mb;

  int count = 0;
  
  while (walk) {
    count++;
    walk = walk->next;
  }
  
  return count;
}

int
gbox_mailbox_list_move(MailBoxList *list, int idx, int direction)
{
  MailBox *box1, *box2;

  box1 = box2 = NULL;
  
  if ((idx == 0 && direction < 0) || 
      (idx == (gbox_mailbox_list_count(list) - 1) && direction > 0) || 
      !direction)
    return FALSE;

  if (direction < 0) {
    box1 = gbox_mailbox_copy(gbox_mailbox_get(list, idx));
    box2 = gbox_mailbox_copy(gbox_mailbox_get(list, idx - 1));
    gbox_mailbox_change(list, box1->mailfile, box2);
    gbox_mailbox_change(list, box2->mailfile, box1);
  } else if (direction > 0) {
    box1 = gbox_mailbox_copy(gbox_mailbox_get(list, idx));
    box2 = gbox_mailbox_copy(gbox_mailbox_get(list, idx + 1));
    gbox_mailbox_change(list, box2->mailfile, box1);
    gbox_mailbox_change(list, box1->mailfile, box2);
  }

  gbox_mailbox_free(box1);
  gbox_mailbox_free(box2);
  
  return TRUE;
} 

void 
gbox_mailbox_free(MailBox *box)
{
    g_free(box->mailfile);
    g_free(box->nickname);

    g_free(box);
}

void
gbox_mailbox_change(MailBoxList *list, char *file, MailBox* new_box)
{
  MailBox *walk = list->first_mb;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    walk = walk->next;
  }
  
  if (walk) {
    g_free(walk->mailfile);
    g_free(walk->nickname);
    walk->mailfile = g_strdup(new_box->mailfile);
    walk->nickname = g_strdup(new_box->nickname);
    walk->size = new_box->size;
    walk->interval = new_box->interval;
    walk->priority = new_box->priority;
    walk->show_in_menu = new_box->show_in_menu;
    walk->mail_count = new_box->mail_count;
    walk->menu_type = new_box->menu_type;
    walk->changed_last_time = new_box->changed_last_time;
    walk->modification_time = new_box->modification_time;
  }
}

void
gbox_mailbox_remove(MailBoxList *list, char *file)
{
  MailBox *walk = list->first_mb, *prev = NULL;
  
  while (walk) {
    if (!strcmp(walk->mailfile, file)) break;
    
    prev = walk;
    walk = walk->next;
  }
  
  if (!walk) return;
  
  if (!prev) {   /* remove first; */
    list->first_mb = list->first_mb->next;
  } else {
    prev->next = walk->next;
  }
  
  if (list->last_mb == walk) list->last_mb = prev;
  
  gbox_mailbox_free(walk);
}

void
gbox_mailbox_remove_by_index(MailBoxList *list, int index)
{
  gbox_mailbox_remove(list, gbox_mailbox_get(list, index)->mailfile);
}

MailBox *
gbox_mailbox_get(MailBoxList *list, int index)
{
  MailBox *walk = list->first_mb;
  int i = 0;
  
  while (walk) {
    if (i == index) break;
    i++;
    walk = walk->next;
  }

  return walk;
}

char **
gbox_mailbox_row_get(MailBox *box)
{
  char **row = (char **)g_malloc(sizeof(char*) * COLUMN_COUNT);
  char buf[256];

  row[0] = g_strdup(box->nickname);
  g_snprintf(buf, sizeof(buf), "%d", box->interval / 1000);
  row[1] = g_strdup(buf);
  g_snprintf(buf, sizeof(buf), "%d", box->priority);
  row[2] = g_strdup(buf);
  if (box->show_in_menu)
    row[3] = g_strdup(_("yes"));
  else
    row[3] = g_strdup(_("no"));

  return row;
}

void
gbox_mailbox_row_free(char **row)
{
  int i;
  
  for (i = 0; i < COLUMN_COUNT; i++)
    if (row[i])
      g_free(row[i]);
    else
      printf("Oops! Were is my row item????\n");
}
