#ifndef GBOX_MAILBOX_H
#define GBOX_MAILBOX_H

#include <sys/types.h>
#include "gbox_defs.h"
#include <glib.h>

#define MENU_TYPE_SUBJECT 0
#define MENU_TYPE_FROM    1
#define MENU_TYPE_BOTH    2

#define COLUMN_COUNT      4

typedef struct _mailbox MailBox;
typedef struct _mailbox_list MailBoxList;
typedef struct _mailentry MailEntry;

struct _mailentry {
    char *from;
      char *subject;
        MailEntry *next;
};

struct _mailbox {
  char *mailfile;
  char *nickname;
  int  size;
  int  interval;
  int  timer;
  int  priority;
  int  show_in_menu;
  int  menu_type;
  int  mail_count;
  time_t modification_time;
  gboolean  changed_last_time;
  MailBox *next;
};

struct _mailbox_list {
  MailBox *first_mb;
  MailBox *last_mb;
  int     new_count[COUNT_PRIORITY];
  int     box_count;
};

MailBoxList *gbox_mailbox_list_new(void);
void gbox_mailbox_list_free(MailBoxList *list);
MailBoxList *gbox_mailbox_list_copy(MailBoxList *list);
int gbox_mailbox_list_count_sum(MailBoxList *list, int prio);
int gbox_mailbox_list_count(MailBoxList *list);
int gbox_mailbox_list_count_with_new_mail(MailBoxList *list);
int gbox_mailbox_list_move(MailBoxList *list, int idx, int direction);

MailBox *gbox_mailbox_new(char *file, char *nick, int interval, int prio, int show, int menu_type);
void gbox_mailbox_free(MailBox *box);
MailBox *gbox_mailbox_copy(MailBox *box);
void gbox_mailbox_add(MailBoxList *list, MailBox *new_box);
void gbox_mailentries_free(MailEntry **mail);
int gbox_mailbox_count_mails(char *file, MailEntry **mail, int with_data);
void gbox_mailbox_change(MailBoxList *list, char *file, MailBox* new_box);
void gbox_mailbox_remove(MailBoxList *list, char *file);
void gbox_mailbox_remove_by_index(MailBoxList *list, int index);
MailBox *gbox_mailbox_get(MailBoxList *list, int index);
char **gbox_mailbox_row_get(MailBox *box);
void gbox_mailbox_row_free(char **row);

#endif
