#include <gnome.h>
#include <libgnomeui/gnome-stock.h>
#include "gbox_mailbox_dlg.h"
#include "gbox_prop.h"
#include "gbox_mailbox.h"

extern GtkWidget      *prop_box;
extern GtkWidget      *file_selector;
extern int            edit_mode;
extern MailBox        *exchange_mb;
extern MailBox        *default_box;
extern MailBoxList    *copy_list;
extern gchar          *menu_types[];
extern PropPageGlobal page_global;
extern int            current_row;

PropPageDefault mailbox_dlg;

GtkWidget *dlg;
int       dlg_mailbox_type;
gchar     *old_mailbox_file;

void
store_mailbox(GtkFileSelection *selector, gpointer user_data)
{
  gtk_entry_set_text(GTK_ENTRY(((PropPageDefault *)user_data)->mailbox), 
		     gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector)));
}

void
select_mailbox_cb(GtkWidget* w, gpointer cbdata)
{
  file_selector = gtk_file_selection_new(_("Select a mailbox"));
    
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_mailbox), cbdata);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);
    
  gtk_widget_show (file_selector);
}

void
menu_changed(GtkWidget* w, gpointer cbdata)
{
  gtk_widget_set_sensitive(GTK_WIDGET(((PropPageDefault *) cbdata)->cbo_menu_type), 
			   gtk_toggle_button_get_active
			   (GTK_TOGGLE_BUTTON(((PropPageDefault *) cbdata)->chk_show_in_menu)));
}

static gint
ok_cb(GtkWidget* w, gpointer cbdata)
{
  gchar   *tmp;
  gchar   **row;
  MailBox *box;
  int     i;
  
  edit_mode = FALSE;
  
  tmp = gtk_editable_get_chars(GTK_EDITABLE(mailbox_dlg.mailbox), 0, -1);
  if (!strlen(tmp)) {
    gnome_error_dialog(_("You have to enter a filename!"));
    return 0;
  }
  exchange_mb->mailfile = g_strdup(tmp);
  g_free(tmp);

  tmp = gtk_editable_get_chars(GTK_EDITABLE(mailbox_dlg.nickname), 0, -1);
  if (!strlen(tmp)) {
    gnome_error_dialog(_("It is better to enter a nickname!"));
    return 0;
  }
  exchange_mb->nickname = g_strdup(tmp);
  g_free(tmp);

  tmp = gtk_editable_get_chars(GTK_EDITABLE(mailbox_dlg.interval), 0, -1);
  if (!strlen(tmp)) {
    gnome_error_dialog(_("You have to enter an interval!"));
    return 0;
  }
  exchange_mb->interval = atoi(tmp) * 1000;
  exchange_mb->show_in_menu = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mailbox_dlg.chk_show_in_menu));

  if (exchange_mb->priority != 
      gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio))) {
    if (exchange_mb->mail_count) {
      copy_list->new_count[exchange_mb->priority]--;
      copy_list->new_count[gtk_spin_button_get_value_as_int
			  (GTK_SPIN_BUTTON(mailbox_dlg.btn_prio))]++;
    }
  }
  exchange_mb->priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio));

  g_free(tmp);

  tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(mailbox_dlg.cbo_menu_type)->entry), 0, -1);
  if (!strcmp(_(menu_types[0]), tmp))
    exchange_mb->menu_type = 0;
  else
    if (!strcmp(_(menu_types[1]), tmp))
      exchange_mb->menu_type = 1;
    else
      if (!strcmp(_(menu_types[2]), tmp))
	exchange_mb->menu_type = 2;

  g_free(tmp);
    
  if (dlg_mailbox_type) {
    box = gbox_mailbox_copy(exchange_mb);
    box->changed_last_time = FALSE;
    box->modification_time = 0;
    box->mail_count = 0;
    box->size = 0;
    gbox_mailbox_add(copy_list, box);
  } else
    gbox_mailbox_change(copy_list, old_mailbox_file, exchange_mb);

  g_free(old_mailbox_file);
  
  row = gbox_mailbox_row_get(exchange_mb);
  
  if (!dlg_mailbox_type) {
    for (i = 0; i < COLUMN_COUNT; i++)
      gtk_clist_set_text(GTK_CLIST(page_global.clist), current_row, i, row[i]);
  } else 
    gtk_clist_append(GTK_CLIST(page_global.clist), row);

  gbox_mailbox_row_free(row);
  
  gtk_widget_destroy(GTK_WIDGET(cbdata));
  gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_box));
  
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_add), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_copy), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_up), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_down), (current_row != -1?TRUE:FALSE) );

  if (exchange_mb) {
    g_free(exchange_mb->mailfile);
    g_free(exchange_mb->nickname); 
  }

  return 0;
}

static gint
cancel_cb(GtkWidget* w, gpointer cbdata)
{
  gtk_widget_destroy(GTK_WIDGET(cbdata));

  edit_mode = FALSE;
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_add), TRUE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_copy), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_up), (current_row != -1?TRUE:FALSE) );
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_down), (current_row != -1?TRUE:FALSE) );

  return 0;
}

void
show_box_dialog()
{
  GtkWidget *table;
  GtkWidget *label;
  gchar     buf[32];
  GtkWidget *ok;
  GtkWidget *box;
  GtkWidget *cancel;
  GtkObject *spin_conf;
  gchar     *text[3];
  MailBox   *tmp_mailbox;
  GList     *list = NULL;

  text[0] = text[1] = text[3] = NULL;
  
  edit_mode = TRUE;
  
  if (!dlg_mailbox_type) {
    tmp_mailbox = gbox_mailbox_get(copy_list, current_row);
    
    exchange_mb = gbox_mailbox_copy(tmp_mailbox);
  }

  dlg = gtk_dialog_new ();
  
  if (!dlg_mailbox_type)
    gtk_window_set_title(GTK_WINDOW(dlg), _("Edit mailbox"));
  else {
    if (dlg_mailbox_type == 1)
      gtk_window_set_title(GTK_WINDOW(dlg), _("Add mailbox"));
    else
      gtk_window_set_title(GTK_WINDOW(dlg), _("Copy mailbox"));
  }
  
  
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_add), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_copy), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_up), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_down), FALSE);
  gtk_window_set_modal(GTK_WINDOW(dlg), FALSE);
  
  table = gtk_table_new(6, 3, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
  
  label = gtk_label_new(_("Mailbox:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  box = gtk_hbox_new(FALSE, 0);
  
  mailbox_dlg.mailbox = gtk_entry_new();
  
  old_mailbox_file = g_strdup(exchange_mb->mailfile);
  
  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    gtk_entry_set_text(GTK_ENTRY(mailbox_dlg.mailbox), exchange_mb->mailfile);
  else  
    gtk_entry_set_text(GTK_ENTRY(mailbox_dlg.mailbox), default_box->mailfile);

  mailbox_dlg.btn_new_mailbox = gtk_button_new_with_label(_("..."));

  gtk_box_pack_start(GTK_BOX(box), mailbox_dlg.mailbox, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box), mailbox_dlg.btn_new_mailbox, FALSE, FALSE, GNOME_PAD_SMALL);

  gtk_table_attach(GTK_TABLE(table), box, 1, 4, 0, 1,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);    
  
  /* Nickname for mailbox*/
  label = gtk_label_new(_("Nickname:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  mailbox_dlg.nickname = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(table), mailbox_dlg.nickname, 1, 3, 1, 2,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    gtk_entry_set_text(GTK_ENTRY(mailbox_dlg.nickname), exchange_mb->nickname);
  else
    gtk_entry_set_text(GTK_ENTRY(mailbox_dlg.nickname), default_box->nickname);

  /* second row */
  label = gtk_label_new(_("Interval:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  box = gtk_hbox_new(FALSE, 0);
  mailbox_dlg.interval = gtk_entry_new();
  label = gtk_label_new(_("sec."));
  gtk_box_pack_start(GTK_BOX(box), mailbox_dlg.interval, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, GNOME_PAD_SMALL);
  
  /*   gtk_entry_set_editable(GTK_ENTRY(interval), FALSE); */
  gtk_table_attach(GTK_TABLE(table), box, 1, 2, 2, 3,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    g_snprintf(buf, sizeof(buf), "%d", exchange_mb->interval / 1000);
  else
    g_snprintf(buf, sizeof(buf), "%d", default_box->interval / 1000);

  gtk_entry_set_text(GTK_ENTRY(mailbox_dlg.interval), buf);
  
  /* third row */
  label = gtk_label_new(_("Priority:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  spin_conf = gtk_adjustment_new(0, 0, COUNT_PRIORITY - 1, 1.0, 1.0, 1.0);
  mailbox_dlg.btn_prio = gtk_spin_button_new(GTK_ADJUSTMENT(spin_conf), 1.0, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio), 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio), TRUE);
  gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio), TRUE);
  
  gtk_table_attach(GTK_TABLE(table), mailbox_dlg.btn_prio, 1, 2, 3, 4,
		   GTK_FILL, 
		   GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio), (float)exchange_mb->priority);
  else
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(mailbox_dlg.btn_prio), (float)default_box->priority);


  mailbox_dlg.chk_show_in_menu = gtk_check_button_new_with_label(_("show mailbox in menu"));

  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mailbox_dlg.chk_show_in_menu), 
				 exchange_mb->show_in_menu);
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(mailbox_dlg.chk_show_in_menu), 
				 default_box->show_in_menu);

  gtk_table_attach(GTK_TABLE(table), mailbox_dlg.chk_show_in_menu, 0, 2, 4, 5,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  mailbox_dlg.cbo_menu_type = gtk_combo_new();
  g_list_free(list); list = NULL;
  
  list = g_list_append(list,_(menu_types[0])); 
  list = g_list_append(list,_(menu_types[1]));  
  list = g_list_append(list,_(menu_types[2]));

  gtk_combo_set_popdown_strings(GTK_COMBO(mailbox_dlg.cbo_menu_type), list); 
  gtk_combo_set_use_arrows(GTK_COMBO(mailbox_dlg.cbo_menu_type), 3);

  if ((!dlg_mailbox_type || dlg_mailbox_type == 2) && exchange_mb)
    tmp_mailbox = exchange_mb;
  else
    tmp_mailbox = default_box;
    
  switch (tmp_mailbox->menu_type) {
  case 0:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mailbox_dlg.cbo_menu_type)->entry), 
		       _(menu_types[0])); break;
  case 1:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mailbox_dlg.cbo_menu_type)->entry),
		       _(menu_types[1])); break;
  case 2:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(mailbox_dlg.cbo_menu_type)->entry), _(menu_types[2]));
  }

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(mailbox_dlg.cbo_menu_type)->entry), FALSE); 
  
  gtk_widget_set_sensitive
    (GTK_WIDGET(mailbox_dlg.cbo_menu_type), 
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mailbox_dlg.chk_show_in_menu)));

  gtk_table_attach(GTK_TABLE(table), mailbox_dlg.cbo_menu_type, 0, 2, 5, 6,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  gtk_signal_connect(GTK_OBJECT (mailbox_dlg.chk_show_in_menu), "toggled",
		     GTK_SIGNAL_FUNC(menu_changed), 
		     &mailbox_dlg);

  /* buttons */
  ok = gnome_stock_button(GNOME_STOCK_BUTTON_OK);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->action_area), ok);
  
  cancel = gnome_stock_button(GNOME_STOCK_BUTTON_CANCEL);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->action_area), cancel);
  
  gtk_widget_show(GTK_WIDGET(ok));
  gtk_widget_show(GTK_WIDGET(cancel));
  
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->vbox), table);
  gtk_widget_show_all(table);
  
  gtk_signal_connect(GTK_OBJECT(mailbox_dlg.btn_new_mailbox), "clicked",
		     GTK_SIGNAL_FUNC(select_mailbox_cb), &mailbox_dlg);
  
  gtk_signal_connect(GTK_OBJECT(ok), "clicked",
		     GTK_SIGNAL_FUNC(ok_cb), dlg);
  
  gtk_signal_connect(GTK_OBJECT(dlg), "destroy",
		     GTK_SIGNAL_FUNC(cancel_cb), dlg);

  gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
		     GTK_SIGNAL_FUNC(cancel_cb), dlg);
  
  gtk_widget_show(dlg);
}
