#include "gbox_prop.h"
#include "gbox_applet.h"
#include "gbox_mailbox.h"
#include "gbox_mailbox_dlg.h"
#include <stdlib.h>
#include <string.h>

extern int     dlg_mailbox_type;
extern gchar   *old_mailbox_file;
extern MailBox *default_box;

GtkWidget *file_selector;
int       edit_mode;

GtkWidget *prop_box;
GtkWidget *file_selector;
GtkWidget *icon_selector[COUNT_PRIORITY + 1];
int       current_row;

PropPageGlobal  page_global;
PropPageMisc    page_misc;
PropPageDefault page_defaults;
Icons           current_icons;
MailBoxList     *copy_list;

gchar *tooltip_types[] = { N_("x/y mailboxes contain new mail"), 
			   N_("sorted by priorities"), 
			   N_("each mailbox separate"),
			   N_("each mailbox with new mail")};

gchar *tooltip_pos[] = {N_("above"),
			N_("to the left"),
			N_("below"),
			N_("to the right")};

gchar *menu_types[] = {N_("<subject>"),
		       N_("<sender>"),
		       N_("<sender> -- <subject>")};

extern MailBox     *exchange_mb;
extern MailBoxList *mb_list;
extern GboxApplet  *applet;

void
store_mailprog(GtkFileSelection *selector, gpointer user_data)
{
  gtk_entry_set_text(GTK_ENTRY(page_global.mailprog), 
		     gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector)));
}

void
select_mailprog(GtkWidget* w, gpointer cbdata)
{
  file_selector = gtk_file_selection_new(_("Select a mail-client"));
    
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (store_mailprog), &page_defaults);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);

  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     (gpointer) file_selector);
    
  gtk_widget_show (file_selector);
}

static gint
box_select_cb(GtkWidget* w, gint row, gint column, 
		GdkEventButton* event, gpointer cbdata)
{
  current_row = row;
  if (!edit_mode) {
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), (row != -1?TRUE:FALSE) );
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), (row != -1?TRUE:FALSE) );
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_copy), (row != -1?TRUE:FALSE) );
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_up), (row != -1 && row != 0?TRUE:FALSE) );
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_down), 
			     (row != -1 && row != (gbox_mailbox_list_count(copy_list) - 1)?TRUE:FALSE));
  }

  return 0;
}

void
move_down_cb(GtkWidget* w, gpointer cbdata)
{
  char **row;
  
  gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_box));
  row = gbox_mailbox_row_get(gbox_mailbox_get(copy_list, current_row));
  gbox_mailbox_list_move(copy_list, current_row, 1);
  gtk_clist_remove(GTK_CLIST(page_global.clist), current_row);

  gtk_clist_insert(GTK_CLIST(page_global.clist), current_row + 1, row);
  gtk_clist_select_row(GTK_CLIST(page_global.clist), current_row + 1, 0);
  gbox_mailbox_row_free(row); 
}

void
move_up_cb(GtkWidget* w, gpointer cbdata)
{
  char **row;

  gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_box));
  row = gbox_mailbox_row_get(gbox_mailbox_get(copy_list, current_row));
  gbox_mailbox_list_move(copy_list, current_row, -1);
  gtk_clist_remove(GTK_CLIST(page_global.clist), current_row);

  gtk_clist_insert(GTK_CLIST(page_global.clist), current_row - 1, row);
  gtk_clist_select_row(GTK_CLIST(page_global.clist), current_row - 1, 0);
  gbox_mailbox_row_free(row);
}
      
void
add_box_cb(GtkWidget* w, gpointer cbdata)
{
  dlg_mailbox_type = 1;
  show_box_dialog();
}

void
edit_box_cb(GtkWidget* w, gpointer cbdata)
{
  dlg_mailbox_type = 0;
  show_box_dialog();  
}

void
remove_or_not(gint reply,gpointer data)
{
  if (!reply) { 
    gbox_mailbox_remove_by_index(copy_list, current_row);
    update_tooltip();
    gtk_clist_remove(GTK_CLIST(page_global.clist), current_row);
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), FALSE );
    gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), FALSE );
    current_row = -1;
    gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_box));
  }
}

void
remove_box_cb(GtkWidget* w, gpointer cbdata)
{
  gchar *text[COLUMN_COUNT];
  gchar *msg;
  int   i;
  
  for (i = 0; i < COLUMN_COUNT; i++) text[0] = NULL;
  
  msg = (gchar *)g_malloc(512);
  
  gtk_clist_get_text(GTK_CLIST(page_global.clist), current_row, 0, text);
  g_snprintf(msg, sizeof(msg), _("Do you really want to remove the mailbox '%s'?"), text[0]);
  gnome_question_dialog_modal(msg, remove_or_not, text[0]);
  g_free(msg);
}

void
copy_box_cb(GtkWidget* w, gpointer cbdata)
{
  MailBox * mailbox = gbox_mailbox_get(copy_list, current_row);
  
  dlg_mailbox_type = 2;
  exchange_mb->mailfile = g_strdup(mailbox->mailfile);
  exchange_mb->nickname = g_strdup(mailbox->nickname);
  exchange_mb->interval = mailbox->interval;
  exchange_mb->priority = mailbox->priority;
  show_box_dialog();  
}


void
text_changed(GtkWidget* w, gpointer cbdata)
{
  gnome_property_box_changed(GNOME_PROPERTY_BOX(prop_box));
  gtk_widget_set_sensitive(GTK_WIDGET(page_misc.cbo_tooltip_type), 
			   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip)));
  gtk_widget_set_sensitive(GTK_WIDGET(page_misc.cbo_tooltip_pos), 
			   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip)));
}


gint
close_prop_cb(GnomeDialog *dialog, gpointer user_data)
{
  int i;
  
  applet->prop_box_active = FALSE;
  start_timer();
  g_free(current_icons.file_no_mail);
  for (i = 0; i < COUNT_PRIORITY; i++)
    g_free(current_icons.file_new_mail[i]);

  gbox_mailbox_list_free(copy_list);
  copy_list = NULL;
  
  return FALSE;
}

void
apply_prop_cb(GtkWidget* propertybox, gint page_num, gpointer data)
{
  gchar         *tmp;
  int           i;
  
  switch (page_num) {
  case 0:
    tmp = gtk_editable_get_chars(GTK_EDITABLE(page_global.mailprog), 0, -1);
    if (applet->mail_prog) g_free(applet->mail_prog);
    applet->mail_prog = g_strdup(tmp);
    g_free(tmp);
    mb_list = gbox_mailbox_list_copy(copy_list);
    break;
  case 1:
    gbox_gfx_change(applet->no_email_image,
		    gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(current_icons.no_mail)));
    
    for (i = 0; i < COUNT_PRIORITY; i++) {
      gbox_gfx_change(applet->new_email_image[i],
		      gnome_icon_entry_get_filename(GNOME_ICON_ENTRY(current_icons.new_mail[i])));
    }

    break;
  case 2:
    applet->style_beep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_beep));

    applet->show_frame = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_frame));
    gtk_frame_set_shadow_type(GTK_FRAME(applet->gbox_frame), 
			      (applet->show_frame?GTK_SHADOW_ETCHED_IN:GTK_SHADOW_NONE));

    applet->show_tooltip = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip));
    tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(page_misc.cbo_tooltip_type)->entry), 0, -1);
    if (!strcmp(_(tooltip_types[0]), tmp))
      applet->style_tooltip = 0;
    else
      if (!strcmp(_(tooltip_types[1]), tmp))
	applet->style_tooltip = 1;
      else
	if (!strcmp(_(tooltip_types[2]), tmp))
	  applet->style_tooltip = 2;
	else
	  if (!strcmp(_(tooltip_types[3]), tmp))
	    applet->style_tooltip = 3;

    g_free(tmp);

    tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry), 0, -1);
    if (!strcmp(_(tooltip_pos[0]), tmp))
      applet->pos_tooltip = 0;
    else
      if (!strcmp(_(tooltip_pos[1]), tmp))
	applet->pos_tooltip = 1;
      else
	if (!strcmp(_(tooltip_pos[2]), tmp))
	  applet->pos_tooltip = 2;
	else
	  if (!strcmp(tooltip_pos[3], tmp))
	    applet->pos_tooltip = 3;

    g_free(tmp);

    applet->show_mail_count = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_mail_count));
    applet->font_name = g_strdup(gnome_font_picker_get_font_name(GNOME_FONT_PICKER(page_misc.font_picker)));
    applet->mail_count_prio = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(page_misc.spin_prio));
    applet->mail_count_x_pos = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(page_misc.spin_x_pos));
    applet->mail_count_y_pos = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(page_misc.spin_y_pos));

    break;
  case 3:
    if (default_box->mailfile) g_free(default_box->mailfile);
    if (default_box->nickname) g_free(default_box->nickname);

    tmp = gtk_editable_get_chars(GTK_EDITABLE(page_defaults.mailbox), 0, -1);
    default_box->mailfile = g_strdup(tmp);
    g_free(tmp);

    tmp = gtk_editable_get_chars(GTK_EDITABLE(page_defaults.nickname), 0, -1);
    default_box->nickname = g_strdup(tmp);
    g_free(tmp);

    tmp = gtk_editable_get_chars(GTK_EDITABLE(page_defaults.interval), 0, -1);

    default_box->interval = atoi(tmp) * 1000;
    default_box->show_in_menu = 
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_defaults.chk_show_in_menu));
    g_free(tmp);

    default_box->priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(page_defaults.btn_prio));

    tmp = gtk_editable_get_chars(GTK_EDITABLE(GTK_COMBO(page_defaults.cbo_menu_type)->entry), 0, -1);
    if (!strcmp(_(menu_types[0]), tmp))
      default_box->menu_type = 0;
    else
      if (!strcmp(_(menu_types[1]), tmp))
	default_box->menu_type = 1;
      else
	if (!strcmp(_(menu_types[2]), tmp))
	  default_box->menu_type = 2;

    g_free(tmp);
  }
}

GtkWidget *
create_page_global()
{
  int        i;
  MailBox    *walk = copy_list->first_mb;
  gchar* col_titles[] = {_("nickname"), _("interval"), _("priority"), _("menu")};
  GtkWidget* table;
  GtkWidget* label;
  GtkWidget* scroll_win;
  char      **row;
  
  current_row = -1;
  
  table = gtk_table_new(8, 5, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
  
  /* third line */
  label = gtk_label_new(_("Mailreader:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   0, 0);
  
  page_global.mailprog = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(table), page_global.mailprog, 1, 4, 2, 3,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   0, GNOME_PAD_SMALL);
  gtk_entry_set_text(GTK_ENTRY(page_global.mailprog), applet->mail_prog);
  
  /*4th line*/
  page_global.btn_move_down = gnome_stock_button(GNOME_STOCK_BUTTON_DOWN);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_down), FALSE);
  page_global.btn_move_up = gnome_stock_button(GNOME_STOCK_BUTTON_UP); 
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.btn_move_up), FALSE);

  gtk_table_attach(GTK_TABLE(table), page_global.btn_move_up, 4, 5, 3, 4,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  gtk_table_set_row_spacing(GTK_TABLE(table), 3, GNOME_PAD_BIG + GNOME_PAD_BIG);
  gtk_table_set_row_spacing(GTK_TABLE(table), 4, GNOME_PAD_BIG + GNOME_PAD_BIG);
  gtk_table_set_row_spacing(GTK_TABLE(table), 5, GNOME_PAD_BIG + GNOME_PAD_BIG);
  
  gtk_table_attach(GTK_TABLE(table), page_global.btn_move_down, 4, 5, 6, 7,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  scroll_win = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll_win),
				 GTK_POLICY_AUTOMATIC,
				 GTK_POLICY_ALWAYS);
  gtk_container_set_border_width(GTK_CONTAINER(scroll_win), 1);
  
  page_global.clist = gtk_clist_new_with_titles(4, col_titles);
  gtk_clist_set_shadow_type(GTK_CLIST(page_global.clist), GTK_SHADOW_IN);
  gtk_clist_set_column_auto_resize (GTK_CLIST(page_global.clist), 0, TRUE);
  
  gtk_clist_set_selection_mode(GTK_CLIST(page_global.clist), GTK_SELECTION_SINGLE);
  
  for (i = 0; i < gbox_mailbox_list_count(copy_list); i++) {
    row = gbox_mailbox_row_get(walk);
    gtk_clist_append(GTK_CLIST(page_global.clist), row);
    gbox_mailbox_row_free(row); 
    walk = walk->next;
  }
  
  gtk_widget_show(page_global.clist);
  gtk_container_add(GTK_CONTAINER(scroll_win), page_global.clist);
  
  gtk_table_attach(GTK_TABLE(table), scroll_win, 0, 4, 3, 7, 
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   0, 0);
  
  page_global.box_add = 
    gnome_pixmap_button
    (GTK_WIDGET(gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_ADD)), _("Add"));

  gtk_table_attach(GTK_TABLE(table), page_global.box_add, 0, 1, 7, 8,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  page_global.box_remove = 
    gnome_pixmap_button
    (GTK_WIDGET(gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_REMOVE)), _("Remove"));

  gtk_table_attach(GTK_TABLE(table), page_global.box_remove, 1, 2, 7, 8,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_remove), FALSE);
  
  page_global.box_edit = 
    gnome_pixmap_button
    (GTK_WIDGET(gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_PREFERENCES)), _("Edit"));

  gtk_table_attach(GTK_TABLE(table), page_global.box_edit, 2, 3, 7, 8,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_edit), FALSE);
  
  page_global.box_copy = 
    gnome_pixmap_button
    (GTK_WIDGET(gnome_stock_new_with_icon(GNOME_STOCK_PIXMAP_COPY)), _("Copy"));

  gtk_table_attach(GTK_TABLE(table), page_global.box_copy, 3, 4, 7, 8,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  gtk_widget_set_sensitive(GTK_WIDGET(page_global.box_copy), FALSE);

  /* show table */
  gtk_widget_show_all(table);
  
  return table;
}

GtkWidget *
create_page_prio() 
{
  GtkWidget *table;
  GtkWidget *label;
  GtkWidget *box;
  GtkWidget *t;
  gchar      buf[256];
  int        i;
  
  table = gtk_table_new(8, 4, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
  
  /*  first line */
  label = gtk_label_new(_("\"No Mail\"-Image:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  current_icons.no_mail = gnome_icon_entry_new("no_mail", _("Select Icon"));
  gnome_icon_entry_set_icon
    (GNOME_ICON_ENTRY(current_icons.no_mail), current_icons.file_no_mail);

  t = gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY (current_icons.no_mail));
  gtk_table_attach(GTK_TABLE(table), current_icons.no_mail, 1, 2, 0, 1,
		   GTK_FILL, GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  
  /* second line */
  label = gtk_label_new(_("\"New Mail\"-Images:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  for (i = 0; i < COUNT_PRIORITY; i++) {
    box = gtk_hbox_new (0, 0);
    g_snprintf(buf, sizeof(buf), "%d:", i);
    label = gtk_label_new(buf);
    g_snprintf(buf, sizeof(buf), "prio_%d_icon_selector", i);
    current_icons.new_mail[i] = gnome_icon_entry_new(buf, _("Select Icon"));
    t = gnome_icon_entry_gtk_entry (GNOME_ICON_ENTRY (current_icons.new_mail[i]));
    
    gnome_icon_entry_set_icon
      (GNOME_ICON_ENTRY(current_icons.new_mail[i]), current_icons.file_new_mail[i]);
    gtk_signal_connect_object_while_alive (GTK_OBJECT (t), "changed",
					   GTK_SIGNAL_FUNC(text_changed), 
					   GTK_OBJECT(prop_box));
    gtk_box_pack_start(GTK_BOX(box), 
		       GTK_WIDGET(label), TRUE, TRUE, GNOME_PAD_SMALL);
    gtk_box_pack_start(GTK_BOX(box), 
		       GTK_WIDGET(current_icons.new_mail[i]), 
		       FALSE, FALSE, GNOME_PAD_SMALL);
    gtk_table_attach(GTK_TABLE(table), box, i+1, i+2, 1, 2,
		     0,
		     0,
		     GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  }
  
  /* show table */
  gtk_widget_show_all(table);
   
  return table;
}

GtkWidget *
create_page_misc()
{
  GtkWidget *table;
  GList     *list = NULL;
  GtkWidget *label;
  GtkWidget *hbox;
  GtkObject *spin_conf;

  table = gtk_table_new(4, 2, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
  
  /*  first line */
  page_misc.chk_beep = gtk_check_button_new_with_label(_("beep on new mail"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page_misc.chk_beep), applet->style_beep);
  gtk_table_attach(GTK_TABLE(table), page_misc.chk_beep, 0, 1, 0, 1,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  /*  second line */
  page_misc.chk_frame = gtk_check_button_new_with_label(_("etched in shadow"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page_misc.chk_frame), applet->show_frame);
  gtk_table_attach(GTK_TABLE(table), page_misc.chk_frame, 0, 2, 1, 2,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  /*  third line */
  page_misc.chk_tooltip = gtk_check_button_new_with_label(_("show tooltip"));

  gtk_table_attach(GTK_TABLE(table), page_misc.chk_tooltip, 0, 1, 2, 3,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  page_misc.cbo_tooltip_type = gtk_combo_new();

  list = g_list_append(list, _(tooltip_types[0]));
  list = g_list_append(list, _(tooltip_types[1]));
  list = g_list_append(list, _(tooltip_types[2]));
  list = g_list_append(list, _(tooltip_types[3]));

  gtk_combo_set_popdown_strings(GTK_COMBO(page_misc.cbo_tooltip_type), list); 
  gtk_combo_set_use_arrows(GTK_COMBO(page_misc.cbo_tooltip_type), 3);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip), applet->show_tooltip);
  switch (applet->style_tooltip) {
  case 0:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_type)->entry),
		       _(tooltip_types[0])); break;
  case 1:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_type)->entry),
		       _(tooltip_types[1])); break;
  case 2:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_type)->entry),
		       _(tooltip_types[2]));break;
  case 3:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_type)->entry),
		       _(tooltip_types[3]));
  }

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_type)->entry), FALSE); 
  
  gtk_widget_set_sensitive(GTK_WIDGET(page_misc.cbo_tooltip_type), 
			   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip)));

  gtk_table_attach(GTK_TABLE(table), page_misc.cbo_tooltip_type, 1, 2, 2, 3,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);


  /*  fourth line */
  label = gtk_label_new(_("relative position"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.25, 0.5);
  gtk_widget_show(label);
  
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  page_misc.cbo_tooltip_pos = gtk_combo_new();
  g_list_free(list); list = NULL;
  
  list = g_list_append(list,_(tooltip_pos[0])); 
  list = g_list_append(list,_(tooltip_pos[1]));  
  list = g_list_append(list,_(tooltip_pos[2]));
  list = g_list_append(list,_(tooltip_pos[3]));

  gtk_combo_set_popdown_strings(GTK_COMBO(page_misc.cbo_tooltip_pos), list); 
  gtk_combo_set_use_arrows(GTK_COMBO(page_misc.cbo_tooltip_pos), 3);

  switch (applet->pos_tooltip) {
  case 0:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry),
		       _(tooltip_pos[0])); break;
  case 1:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry),
		       _(tooltip_pos[1])); break;
  case 2:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry),
		       _(tooltip_pos[2]));break;
  case 3:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry),
		       _(tooltip_pos[3]));
  }

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(page_misc.cbo_tooltip_pos)->entry), FALSE); 
  
  gtk_widget_set_sensitive(GTK_WIDGET(page_misc.cbo_tooltip_pos), 
			   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_misc.chk_tooltip)));

  gtk_table_attach(GTK_TABLE(table), page_misc.cbo_tooltip_pos, 1, 2, 3, 4,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  /* Gnome Font Picker */
  page_misc.chk_mail_count = gtk_check_button_new_with_label(_("show sum of new mails"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page_misc.chk_mail_count), applet->show_mail_count);
  gtk_table_attach(GTK_TABLE(table), page_misc.chk_mail_count, 0, 1, 4, 5,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  page_misc.font_picker = gnome_font_picker_new();
  gtk_table_attach(GTK_TABLE(table), page_misc.font_picker, 1, 2, 4, 5,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(page_misc.font_picker),
					     TRUE,
					     12);

  if (applet->font_name)
    gnome_font_picker_set_font_name (GNOME_FONT_PICKER(page_misc.font_picker),
				     applet->font_name);
  
  label = gtk_label_new(_("priorities to count:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.25, 0.5);
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  

  spin_conf = gtk_adjustment_new(0, 0, COUNT_PRIORITY - 1, 1.0, 1.0, 1.0);
  page_misc.spin_prio = gtk_spin_button_new(GTK_ADJUSTMENT(spin_conf), 1.0, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(page_misc.spin_prio), 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(page_misc.spin_prio), TRUE);
  gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(page_misc.spin_prio), TRUE);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(page_misc.spin_prio), (float)applet->mail_count_prio);

  gtk_table_attach(GTK_TABLE(table), page_misc.spin_prio, 1, 2, 5, 6,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  hbox = gtk_hbox_new(TRUE, 5);
  
  /* x position of mail count*/
  spin_conf = gtk_adjustment_new(0, 0, gbox_gfx_width(applet->new_email_image[0]), 1.0, 1.0, 1.0);
  page_misc.spin_x_pos = gtk_spin_button_new(GTK_ADJUSTMENT(spin_conf), 1.0, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(page_misc.spin_x_pos), 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(page_misc.spin_x_pos), TRUE);
  gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(page_misc.spin_x_pos), TRUE);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(page_misc.spin_x_pos), (float)applet->mail_count_x_pos);
  
  label = gtk_label_new(_("x:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.25, 0.5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), page_misc.spin_x_pos);

  /* y position of mail count*/
  spin_conf = gtk_adjustment_new(0, 0, gbox_gfx_width(applet->new_email_image[0]), 1.0, 1.0, 1.0);
  page_misc.spin_y_pos = gtk_spin_button_new(GTK_ADJUSTMENT(spin_conf), 1.0, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(page_misc.spin_y_pos), 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(page_misc.spin_y_pos), TRUE);
  gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(page_misc.spin_y_pos), TRUE);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(page_misc.spin_y_pos), (float)applet->mail_count_y_pos);

  label = gtk_label_new(_("y:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.25, 0.5);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), label);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), page_misc.spin_y_pos);

  gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 6, 7,
		   GTK_FILL,
		   GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  /* setting up signal-handler */
  gtk_signal_connect_object_while_alive (GTK_OBJECT (page_misc.chk_beep), "toggled",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (page_misc.chk_frame), "toggled",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (page_misc.chk_mail_count), "toggled",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (page_misc.font_picker), "clicked",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (page_misc.chk_tooltip), "toggled",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (GTK_COMBO(page_misc.cbo_tooltip_type)->entry),
					 "changed",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect_object_while_alive (GTK_OBJECT (GTK_COMBO(page_misc.cbo_tooltip_pos)->entry),
					 "changed",
					 GTK_SIGNAL_FUNC(text_changed), 
					 GTK_OBJECT(prop_box));
  gtk_signal_connect(GTK_OBJECT(GTK_EDITABLE(&(GTK_SPIN_BUTTON(page_misc.spin_prio)->entry))),
		     "changed", GTK_SIGNAL_FUNC(text_changed), NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_EDITABLE(&(GTK_SPIN_BUTTON(page_misc.spin_x_pos)->entry))),
		     "changed", GTK_SIGNAL_FUNC(text_changed), NULL);
  gtk_signal_connect(GTK_OBJECT(GTK_EDITABLE(&(GTK_SPIN_BUTTON(page_misc.spin_y_pos)->entry))),
		     "changed", GTK_SIGNAL_FUNC(text_changed), NULL);

  /* show table */
  gtk_widget_show_all(table);

  return table;
}

GtkWidget *
create_page_defaults()
{
  GtkWidget *table;
  GtkWidget *label;
  gchar     buf[32];
  GtkWidget *box;
  GtkObject *spin_conf;
  gchar     *text[3];
  GList     *list = NULL;

  text[0] = text[1] = text[3] = NULL;

  table = gtk_table_new(6, 3, FALSE);
  gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
  
  label = gtk_label_new(_("Mailbox:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  box = gtk_hbox_new(FALSE, 0);
  
  page_defaults.mailbox = gtk_entry_new();
  
  gtk_entry_set_text(GTK_ENTRY(page_defaults.mailbox), default_box->mailfile);
  
  page_defaults.btn_new_mailbox = gtk_button_new_with_label(_("..."));

  gtk_box_pack_start(GTK_BOX(box), page_defaults.mailbox, TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(box), page_defaults.btn_new_mailbox, FALSE, FALSE, GNOME_PAD_SMALL);

  gtk_table_attach(GTK_TABLE(table), box, 1, 4, 0, 1,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);    
  
  /* Nickname for mailbox*/
  label = gtk_label_new(_("Nickname:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  page_defaults.nickname = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(table), page_defaults.nickname, 1, 3, 1, 2,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  gtk_entry_set_text(GTK_ENTRY(page_defaults.nickname), default_box->nickname);

  /* second row */
  
  label = gtk_label_new(_("Interval:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  box = gtk_hbox_new(FALSE, 0);
  page_defaults.interval = gtk_entry_new();
  label = gtk_label_new(_("sec."));
  gtk_box_pack_start(GTK_BOX(box), page_defaults.interval, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, GNOME_PAD_SMALL);
  gtk_table_attach(GTK_TABLE(table), box, 1, 2, 2, 3,
		   GTK_FILL | GTK_EXPAND, 
		   GTK_FILL | GTK_EXPAND, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  g_snprintf(buf, sizeof(buf), "%d", default_box->interval / 1000);
  gtk_entry_set_text(GTK_ENTRY(page_defaults.interval), buf);
  
  /* third row */
  label = gtk_label_new(_("Priority:"));
  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 
		   GTK_EXPAND | GTK_FILL, 
		   GTK_EXPAND | GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  spin_conf = gtk_adjustment_new(0, 0, COUNT_PRIORITY - 1, 1.0, 1.0, 1.0);
  page_defaults.btn_prio = gtk_spin_button_new(GTK_ADJUSTMENT(spin_conf), 1.0, 1);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(page_defaults.btn_prio), 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(page_defaults.btn_prio), TRUE);
  gtk_spin_button_set_snap_to_ticks(GTK_SPIN_BUTTON(page_defaults.btn_prio), TRUE);
  gtk_table_attach(GTK_TABLE(table), page_defaults.btn_prio, 1, 2, 3, 4,
		   GTK_FILL, 
		   GTK_FILL, 
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);
  
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(page_defaults.btn_prio), (float)default_box->priority);

  page_defaults.chk_show_in_menu = gtk_check_button_new_with_label(_("show mailbox in menu"));
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page_defaults.chk_show_in_menu), 
			       default_box->show_in_menu);

  gtk_table_attach(GTK_TABLE(table), page_defaults.chk_show_in_menu, 0, 2, 4, 5,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  page_defaults.cbo_menu_type = gtk_combo_new();
  g_list_free(list); list = NULL;
  
  list = g_list_append(list,_(menu_types[0])); 
  list = g_list_append(list,_(menu_types[1]));  
  list = g_list_append(list,_(menu_types[2]));

  gtk_combo_set_popdown_strings(GTK_COMBO(page_defaults.cbo_menu_type), list); 
  gtk_combo_set_use_arrows(GTK_COMBO(page_defaults.cbo_menu_type), 3);

  switch (default_box->menu_type) {
  case 0:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_defaults.cbo_menu_type)->entry), 
		       _(menu_types[0])); break;
  case 1:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_defaults.cbo_menu_type)->entry),
		       _(menu_types[1])); break;
  case 2:
    gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(page_defaults.cbo_menu_type)->entry),
		       _(menu_types[2]));
  }

  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(page_defaults.cbo_menu_type)->entry), FALSE); 
  
  gtk_widget_set_sensitive
    (GTK_WIDGET(page_defaults.cbo_menu_type), 
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page_defaults.chk_show_in_menu)));

  gtk_table_attach(GTK_TABLE(table), page_defaults.cbo_menu_type, 0, 2, 5, 6,
		   GTK_EXPAND | GTK_FILL,
		   GTK_EXPAND | GTK_FILL,
		   GNOME_PAD_SMALL, GNOME_PAD_SMALL);

  gtk_signal_connect(GTK_OBJECT(page_defaults.mailbox), "changed",
		     GTK_SIGNAL_FUNC(text_changed), NULL);
  
  gtk_signal_connect(GTK_OBJECT(page_defaults.nickname), "changed",
		     GTK_SIGNAL_FUNC(text_changed), NULL);
  
  gtk_signal_connect(GTK_OBJECT(page_defaults.interval), "changed",
		     GTK_SIGNAL_FUNC(text_changed), NULL);
  
  gtk_signal_connect(GTK_OBJECT(GTK_EDITABLE(GTK_COMBO(page_defaults.cbo_menu_type)->entry)),
		     "changed", GTK_SIGNAL_FUNC(text_changed), NULL);

  gtk_signal_connect(GTK_OBJECT(GTK_EDITABLE(&(GTK_SPIN_BUTTON(page_defaults.btn_prio)->entry))),
		     "changed", GTK_SIGNAL_FUNC(text_changed), NULL);

  gtk_signal_connect(GTK_OBJECT(page_defaults.chk_show_in_menu), "toggled",
		     GTK_SIGNAL_FUNC(text_changed), NULL);

  gtk_signal_connect(GTK_OBJECT (page_defaults.chk_show_in_menu), "toggled",
		     GTK_SIGNAL_FUNC(menu_changed), 
		     &page_defaults);

  gtk_signal_connect(GTK_OBJECT(page_defaults.btn_new_mailbox), "clicked",
		     GTK_SIGNAL_FUNC(select_mailbox_cb), &page_defaults);

  gtk_widget_show_all(table);

  return table;
}

void 
mail_items(AppletWidget *app, gpointer data)
{
}

void
applet_properties(AppletWidget *app, gpointer data)
{
  GtkWidget *page;
  int       i;
  
  /* check if property-box is open */
  if (applet->prop_box_active)
    return;
  else
    applet->prop_box_active = TRUE;

  /* stop watch mailboxes; */
  stop_timer();

  edit_mode = FALSE;
  copy_list = gbox_mailbox_list_copy(mb_list);
  
  prop_box = gnome_property_box_new();
   
  gtk_window_set_title(GTK_WINDOW(prop_box), _("gbox properties"));

  /* copy current icon selection */
  current_icons.file_no_mail = g_strdup(gbox_gfx_file(applet->no_email_image));
  for (i = 0; i < COUNT_PRIORITY; i++)
    current_icons.file_new_mail[i] = g_strdup(gbox_gfx_file(applet->new_email_image[i]));
    
  page = create_page_global();
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_box), page,
				 gtk_label_new(_("Mailboxes")));

  page = create_page_prio();
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_box), page,
				 gtk_label_new(_("Icons")));

  page = create_page_misc();
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_box), page,
				 gtk_label_new(_("Misc")));

  page = create_page_defaults();
  gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_box), page,
				 gtk_label_new(_("Default values")));

  gtk_signal_connect(GTK_OBJECT(page_global.mailprog), "changed",
		     GTK_SIGNAL_FUNC(text_changed), prop_box);
  
  gtk_signal_connect(GTK_OBJECT(page_global.clist), "select_row",
		     GTK_SIGNAL_FUNC(box_select_cb), prop_box);
  
  gtk_signal_connect(GTK_OBJECT(page_global.box_edit), "clicked",
		     GTK_SIGNAL_FUNC(edit_box_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(page_global.box_add), "clicked",
		     GTK_SIGNAL_FUNC(add_box_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(page_global.box_remove), "clicked",
		     GTK_SIGNAL_FUNC(remove_box_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(page_global.box_copy), "clicked",
		     GTK_SIGNAL_FUNC(copy_box_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(page_global.btn_move_down), "clicked",
		     GTK_SIGNAL_FUNC(move_down_cb), NULL);
  gtk_signal_connect(GTK_OBJECT(page_global.btn_move_up), "clicked",
		     GTK_SIGNAL_FUNC(move_up_cb), NULL);
  
  gtk_signal_connect(GTK_OBJECT(prop_box), "apply",
		     GTK_SIGNAL_FUNC(apply_prop_cb), NULL);

  gtk_signal_connect(GTK_OBJECT(prop_box), "close",
		     GTK_SIGNAL_FUNC(close_prop_cb), NULL);

  gtk_widget_show(prop_box);
}

